/* ==========================================================
 * Program: 06-StationaryPointsPi2.cpp
 * ==========================================================
 * The program finds stationary points for Poincare map P 
 * or its second iteration P^2 on the x=pi/2 line of symmetry
 * via interval Newton method.
 * The result are small intervals of the form
 * {pi/2, [a,b]}, which contain stationary points.
 * ==========================================================*/ 

#include <iostream>
using namespace std;

#include "capd/capdlib.h"
using namespace capd;
using namespace capd::alglib;
using namespace capd::matrixAlgorithms;

#include "utils.h"

// Main function
void SiatkaNaOsiSym()
{	   	
	mojUklad hyperion;					// for structure mojUklad see utils
	//mojUklad hyperion(2);				// use for second iteration of P
	SecMap Poin(hyperion.P);			// hyperion.P is the full Poincare map, 
										// Poin is its restriction to the z=0 (mod 2pi) section
	PoincNaOsiSym P(Poin);				// P is a self-map of (x=pi/2) line of the form y -> P(pi/2, y, 0)[0] - pi/2,
										// its zeroes are the stationary points on the {x=pi/2} line.
	
	// Preparation of the (pi/2,[0, 2.6])-interval grid
	interval x, X;	
	interval DP, Px, xPop;				
	double iks;							// numerator
	int n=3;							// grid thickness: 10^-n
	double delta = pow(10.0,-n);		
	
	// Iteration of small intervals
	int a;								// numerator
	
	// Table of points
	const int max_punktow = 100;			// I expect no more than 100 interesting points
	interval Tablica_punktow[max_punktow];	// table of points
	int nr_ostatniego_punktu;				// the last point numerator
	
	while(iks*delta < 2.6) 		
	{
		a=1;
		x=iks*delta;
        try{
			do{								// Preliminary check
				Px=P(x,DP);
				xPop = x;
				x=x-Px/DP;
				x=x.mid();				
				a++;
			} while(not(subset(x,xPop)) && a<20); // check if the iteration is 'stable'
			
			// if x is a good candidate, then we try a proper interval Newton method:
			X=x;
			X=X+delta*interval(-1.0,1.0);
           
			interval N=x-P(x)/P[X];		// interval Newton

			if(subsetInterior(N,X))					// is N contained in the primary neighbourhood?
			{
				bool czy_juz_jest = 0;
				for(int i =0; i<=nr_ostatniego_punktu ; i++)		//check if the point is new
				{
					if(not ((N > Tablica_punktow[i]) or (N< Tablica_punktow[i])))			
					{						
						czy_juz_jest = 1;			// it is already in the table.
					}									
				}
				if (czy_juz_jest ==	0) 				// if it is not in the table, we add it
				{
					Tablica_punktow[nr_ostatniego_punktu]= N;
					nr_ostatniego_punktu++;
				}
			}
			else
			{
				//cout << "For the starting point " << iks*pow(10.0,-n) << " no success." << endl;
			}
			} catch(exception& e)
			{
				cout << "\nFor the starting point " << iks*pow(10.0,-n) << " exception caught: " << e.what() << endl;
            }
		iks+=1;
	}
	cout << "\nAll the stationary points for P found on the (pi/2,[0, 2.6])-interval:" << endl;
	for (int i=0;i<nr_ostatniego_punktu; i++)
	{
		cout << "\n" << i <<": "<< Tablica_punktow[i] << endl;
		IVector Y(2);
		Y[0]=2.0*atan(1.0);
		Y[1]=Tablica_punktow[i];
		IMatrix U=Poin[Y];
		cout << "Det = " << Det(U) << ", Tr = " << Tr(U) << ", Char. poly. Delta = " << Delta(U) << endl;			
		if(Tr(Poin[Y])>2)
		{
			interval l1 = (Tr(U)+sqrt(Delta(U)))*0.5;					// eigenvalues
			interval l2 = (Tr(U)-sqrt(Delta(U)))*0.5;
			IVector v1(2);												// eigenvectors
			v1[0]=1;
			v1[1]=(l1-U[0][0])/U[0][1];
			IVector v2(2);
			v2[0]=1;
			v2[1]=(l2-U[0][0])/U[0][1];
			IVector v1N(2);
			v1N=v1/sqrt(power(v1[0],2.0)+power(v1[1],2.0));				// normalization
			IVector v2N(2);
			v2N=v2/sqrt(power(v2[0],2.0)+power(v2[1],2.0));							
			cout << "HYPERBOLIC POINT" << endl;
			cout << "eigenvalues:" << l1 << ", " << l2 <<  endl;
			cout << "eigenvectors:" << v1N << ", " << v2N <<  endl;
		}
	}
return;
}


int main(int argc, char* argv[])
{
  	cout.precision(16);
	try
	{
		SiatkaNaOsiSym();			
	}
	catch(exception& e)
  	{
    		cout << "\n\nException caught: "<< e.what() << endl;
  	}
  return 0;
} 

/* OUTPUT:
 * 
All the stationary points for P found on the (pi/2,[0, 2.6])-interval:

0: [0.2184153641989348, 0.2184153641990009]
Det = [0.9999999999768495, 1.000000000023149], Tr = [2.000119678798856, 2.000119678821127], Char. poly. Delta = [0.0004787294258408181, 0.00047872970012941]
HYPERBOLIC POINT
eigenvalues:[1.010999782564737, 1.010999785709899], [0.9891198931000921, 0.9891198962452538]
eigenvectors:{[0.9999970200120211, 0.9999970200137387],[0.002441303676887635, 0.002441304380175901]}, {[0.9999970200120211, 0.9999970200137387],[-0.002441304380174389, -0.002441303676886173]}

1: [0.8712620245643015, 0.8712620245647508]
Det = [0.999999999901018, 1.000000000098981], Tr = [-0.1164521869723555, -0.1164521867844378], Char. poly. Delta = [-3.986438888589046, -3.986438887753427]

2: [1.099105177037064, 1.099105177037082]
Det = [0.9999999669380486, 1.000000033058313], Tr = [263.7674228147225, 263.7674228149732], Char. poly. Delta = [69569.25333818837, 69569.2533385851]
HYPERBOLIC POINT
eigenvalues:[263.7636315413494, 263.7636315418508], [0.003791272997034412, 0.003791273498478631]
eigenvectors:{[0.7064565371793425, 0.7064565371812066],[0.7077564277852203, 0.7077564277908148]}, {[0.7064565371793425, 0.7064565371812066],[-0.7077564277908146, -0.7077564277852203]}

3: [1.294219468594181, 1.294219468594242]
Det = [0.999999998488903, 1.00000000151104], Tr = [26.84613256989356, 26.84613257000613], Char. poly. Delta = [716.7148339542557, 716.7148339723885]
HYPERBOLIC POINT
eigenvalues:[26.80883142888099, 26.8088314291066], [0.03730114084324399, 0.03730114106886262]
eigenvectors:{[0.7338248323735296, 0.7338248323818986],[0.6793387338943307, 0.6793387339188653]}, {[0.7338248323735297, 0.7338248323818987],[-0.6793387339188652, -0.6793387338943306]}

4: [1.711370591161749, 1.711370591161875]
Det = [0.9999999998594928, 1.000000000140509], Tr = [5.392491851568614, 5.392491851620629], Char. poly. Delta = [25.07896836867187, 25.07896837035691]
HYPERBOLIC POINT
eigenvalues:[5.200191231131044, 5.200191231241172], [0.1923006203534499, 0.1923006204635773]
eigenvectors:{[0.8831685403716123, 0.8831685403830619],[0.469055784818765, 0.4690557848524839]}, {[0.8831685403716135, 0.8831685403830631],[-0.4690557848524816, -0.4690557848187629]}

5: [2.276456591892877, 2.276456591893094]
Det = [0.9999999999285024, 1.000000000071498], Tr = [2.573755865836176, 2.573755865891394], Char. poly. Delta = [2.624219256640131, 2.624219257496352]
HYPERBOLIC POINT
eigenvalues:[2.096850039944023, 2.09685004010377], [0.4769058257600154, 0.4769058259197623]
eigenvectors:{[0.9841366887472078, 0.984136688754278],[0.177411887559187, 0.1774118876009514]}, {[0.9841366887472089, 0.984136688754279],[-0.1774118876009451, -0.1774118875591808]}

6: [2.83845879539493, 2.838458795395145]
Det = [0.9999999999367359, 1.000000000063267], Tr = [2.103975469246583, 2.103975469306356], Char. poly. Delta = [0.4267127749383102, 0.4267127756959583]
HYPERBOLIC POINT
eigenvalues:[1.378604013664275, 1.378604013984123], [0.7253714552923467, 0.7253714556121947]
eigenvectors:{[0.9980008387968913, 0.9980008388010421],[0.06320067841711649, 0.06320067848317479]}, {[0.9980008387968913, 0.9980008388010421],[-0.06320067848317446, -0.06320067841711617]}


*/
